# -*- coding: utf-8 -*-
# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest

from compare_locales.tests import ParserTestMixin
from compare_locales.parser import (
    Comment,
    IniSection,
    Junk,
    Whitespace,
)


mpl2 = '''\
; This Source Code Form is subject to the terms of the Mozilla Public
; License, v. 2.0. If a copy of the MPL was not distributed with this file,
; You can obtain one at http://mozilla.org/MPL/2.0/.'''


class TestIniParser(ParserTestMixin, unittest.TestCase):

    filename = 'foo.ini'

    def testSimpleHeader(self):
        self._test('''; This file is in the UTF-8 encoding
[Strings]
TitleText=Some Title
''', (
            (Comment, 'UTF-8 encoding'),
            (Whitespace, '\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n')))

    def testMPL2_Space_UTF(self):
        self._test(mpl2 + '''

; This file is in the UTF-8 encoding
[Strings]
TitleText=Some Title
''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (Comment, 'UTF-8'),
            (Whitespace, '\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n')))

    def testMPL2_Space(self):
        self._test(mpl2 + '''

[Strings]
TitleText=Some Title
''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n')))

    def testMPL2_MultiSpace(self):
        self._test(mpl2 + '''

; more comments

[Strings]
TitleText=Some Title
''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (Comment, 'more comments'),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n')))

    def testMPL2_JunkBeforeCategory(self):
        self._test(mpl2 + '''
Junk
[Strings]
TitleText=Some Title
''', (
            (Comment, mpl2),
            (Whitespace, '\n'),
            (Junk, 'Junk\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n')))

    def test_TrailingComment(self):
        self._test(mpl2 + '''

[Strings]
TitleText=Some Title
;Stray trailing comment
''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n'),
            (Comment, 'Stray trailing'),
            (Whitespace, '\n')))

    def test_SpacedTrailingComments(self):
        self._test(mpl2 + '''

[Strings]
TitleText=Some Title

;Stray trailing comment
;Second stray comment

''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n\n'),
            (Comment, 'Second stray comment'),
            (Whitespace, '\n\n')))

    def test_TrailingCommentsAndJunk(self):
        self._test(mpl2 + '''

[Strings]
TitleText=Some Title

;Stray trailing comment
Junk
;Second stray comment

''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n\n'),
            (Comment, 'Stray trailing'),
            (Whitespace, '\n'),
            (Junk, 'Junk\n'),
            (Comment, 'Second stray comment'),
            (Whitespace, '\n\n')))

    def test_JunkInbetweenEntries(self):
        self._test(mpl2 + '''

[Strings]
TitleText=Some Title

Junk

Good=other string
''', (
            (Comment, mpl2),
            (Whitespace, '\n\n'),
            (IniSection, 'Strings'),
            (Whitespace, '\n'),
            ('TitleText', 'Some Title'),
            (Whitespace, '\n\n'),
            (Junk, 'Junk\n\n'),
            ('Good', 'other string'),
            (Whitespace, '\n')))

    def test_empty_file(self):
        self._test('', tuple())
        self._test('\n', ((Whitespace, '\n'),))
        self._test('\n\n', ((Whitespace, '\n\n'),))
        self._test(' \n\n', ((Whitespace, ' \n\n'),))


if __name__ == '__main__':
    unittest.main()



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import unittest
from compare_locales.merge import merge_channels


class TestMergeFluent(unittest.TestCase):
    name = "foo.ftl"

    def test_no_changes(self):
        channels = (b"""
foo = Foo 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1
""")

    def test_attribute_in_first(self):
        channels = (b"""
foo = Foo 1
    .attr = Attr 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1
    .attr = Attr 1
""")

    def test_attribute_in_last(self):
        channels = (b"""
foo = Foo 1
""", b"""
foo = Foo 2
    .attr = Attr 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1
""")

    def test_attribute_changed(self):
        channels = (b"""
foo = Foo 1
    .attr = Attr 1
""", b"""
foo = Foo 2
    .attr = Attr 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1
    .attr = Attr 1
""")

    def test_group_comment_in_first(self):
        channels = (b"""
## Group Comment 1
foo = Foo 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
## Group Comment 1
foo = Foo 1
""")

    def test_group_comment_in_last(self):
        channels = (b"""
foo = Foo 1
""", b"""
## Group Comment 2
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
## Group Comment 2
foo = Foo 1
""")

    def test_group_comment_changed(self):
        channels = (b"""
## Group Comment 1
foo = Foo 1
""", b"""
## Group Comment 2
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
## Group Comment 2
## Group Comment 1
foo = Foo 1
""")

    def test_group_comment_and_section(self):
        channels = (b"""
## Group Comment
foo = Foo 1
""", b"""
// Section Comment
[[ Section ]]
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
// Section Comment
[[ Section ]]
## Group Comment
foo = Foo 1
""")

    def test_message_comment_in_first(self):
        channels = (b"""
# Comment 1
foo = Foo 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
# Comment 1
foo = Foo 1
""")

    def test_message_comment_in_last(self):
        channels = (b"""
foo = Foo 1
""", b"""
# Comment 2
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1
""")

    def test_message_comment_changed(self):
        channels = (b"""
# Comment 1
foo = Foo 1
""", b"""
# Comment 2
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
# Comment 1
foo = Foo 1
""")

    def test_standalone_comment_in_first(self):
        channels = (b"""
foo = Foo 1

# Comment 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1

# Comment 1
""")

    def test_standalone_comment_in_last(self):
        channels = (b"""
foo = Foo 1
""", b"""
foo = Foo 2

# Comment 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1

# Comment 2
""")

    def test_standalone_comment_changed(self):
        channels = (b"""
foo = Foo 1

# Comment 1
""", b"""
foo = Foo 2

# Comment 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
foo = Foo 1

# Comment 2

# Comment 1
""")

    def test_resource_comment_in_first(self):
        channels = (b"""
### Resource Comment 1

foo = Foo 1
""", b"""
foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
### Resource Comment 1

foo = Foo 1
""")

    def test_resource_comment_in_last(self):
        channels = (b"""
foo = Foo 1
""", b"""
### Resource Comment 1

foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
### Resource Comment 1

foo = Foo 1
""")

    def test_resource_comment_changed(self):
        channels = (b"""
### Resource Comment 1

foo = Foo 1
""", b"""
### Resource Comment 2

foo = Foo 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
### Resource Comment 2

### Resource Comment 1

foo = Foo 1
""")

    def test_cross_grammar(self):
        channels = (b"""
# Comment 1
foo =
    .attr = Attribute 1
""", b"""
// Comment 2
foo
    .attr = Attribute 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
# Comment 1
foo =
    .attr = Attribute 1
""")

    def test_cross_grammar_standalone_comment(self):
        '''This is in particular going to be triggered for license headers.'''
        channels = (b"""
# Same comment

foo =
    .attr = Attribute 1
""", b"""
// Same comment

foo
    .attr = Attribute 2
""")
        self.assertEqual(
            merge_channels(self.name, *channels), b"""
# Same comment

foo =
    .attr = Attribute 1
""")



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.
from __future__ import print_function
import os
import sys
from mozbuild.util import FileAvoidWrite

header_template = '''#pragma GCC system_header
#pragma GCC visibility push(default)
#include_next <%(header)s>
#pragma GCC visibility pop
'''


# The 'unused' arg is the output file from the file_generate action. We actually
# generate all the files in header_list
def gen_wrappers(unused, outdir, *header_list):
    for header in header_list:
        with FileAvoidWrite(os.path.join(outdir, header)) as f:
            f.write(header_template % {
                'header': header,
            })



# -*- coding: utf-8 -*-

# Any copyright is dedicated to the Public Domain.
# http://creativecommons.org/publicdomain/zero/1.0/

from __future__ import unicode_literals

import os
import unittest

import mozunit

from mozbuild.action.package_fennec_apk import (
    package_fennec_apk as package,
)
from mozpack.mozjar import JarReader
import mozpack.path as mozpath


test_data_path = mozpath.abspath(mozpath.dirname(__file__))
test_data_path = mozpath.join(test_data_path, 'data', 'package_fennec_apk')


def data(name):
    return os.path.join(test_data_path, name)


class TestPackageFennecAPK(unittest.TestCase):
    """
    Unit tests for package_fennec_apk.py.
    """

    def test_arguments(self):
        # Language repacks take updated resources from an ap_ and pack them
        # into an apk.  Make sure the second input overrides the first.
        jarrer = package(inputs=[],
                         omni_ja=data('omni.ja'),
                         classes_dex=data('classes.dex'),
                         assets_dirs=[data('assets')],
                         lib_dirs=[data('lib')],
                         root_files=[data('root_file.txt')])

        # omni.ja ends up in assets/omni.ja.
        self.assertEquals(jarrer['assets/omni.ja'].open().read().strip(), 'omni.ja')

        # Everything else is in place.
        for name in ('classes.dex',
                     'assets/asset.txt',
                     'lib/lib.txt',
                     'root_file.txt'):
            self.assertEquals(jarrer[name].open().read().strip(), name)

    def test_inputs(self):
        # Language repacks take updated resources from an ap_ and pack them
        # into an apk.  In this case, the first input is the original package,
        # the second input the update ap_.  Make sure the second input
        # overrides the first.
        jarrer = package(inputs=[data('input2.apk'), data('input1.ap_')])

        files1 = JarReader(data('input1.ap_')).entries.keys()
        files2 = JarReader(data('input2.apk')).entries.keys()
        for name in files2:
            self.assertTrue(name in files1 or
                            jarrer[name].open().read().startswith('input2/'))
        for name in files1:
            self.assertTrue(jarrer[name].open().read().startswith('input1/'))


if __name__ == '__main__':
    mozunit.main()
